@@ -4,12 +4,14 @@
  * SPDX-License-Identifier: MIT
  */
 
+#include "minimum_runtime_check.js"
+
 #if MODULARIZE
 var Module = moduleArg;
 #elif USE_CLOSURE_COMPILER
 /** @type{Object} */
 var Module;
-// if (!Module)` is crucial for Closure Compiler here as it will
+// if (!Module) is crucial for Closure Compiler here as it will
 // otherwise replace every `Module` occurrence with the object below
 if (!Module) /** @suppress{checkTypes}*/Module = 
 #if AUDIO_WORKLET
@@ -21,14 +23,9 @@ if (!Module) /** @suppress{checkTypes}*/Module =
 
 // When running on the web we expect Module to be defined externally, in the
 // HTML.  Otherwise we must define it here before its first use
-var Module =
-#if SUPPORTS_GLOBALTHIS
-  // As a small code size optimization, we can use 'globalThis' to refer to the global scope Module variable.
-  globalThis.{{{ EXPORT_NAME }}} || {};
-#else
-  // Otherwise do a good old typeof check.
-  typeof {{{ EXPORT_NAME }}} != 'undefined' ? {{{ EXPORT_NAME }}} : {};
-#endif
+// As a small code size optimization, we can use 'globalThis' to refer to the
+// global scope Module variable.
+var Module = globalThis.{{{ EXPORT_NAME }}} || {};
 
 #else
 var Module = {{{ EXPORT_NAME }}};
@@ -39,7 +36,7 @@ var ENVIRONMENT_IS_NODE = {{{ nodeDetectionCode() }}};
 #endif
 
 #if ENVIRONMENT_MAY_BE_SHELL
-var ENVIRONMENT_IS_SHELL = typeof read == 'function';
+var ENVIRONMENT_IS_SHELL = !!globalThis.read;
 #endif
 
 #if ASSERTIONS || PTHREADS
@@ -58,31 +55,32 @@ var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_NODE;
 
 #if ENVIRONMENT_MAY_BE_NODE && (PTHREADS || WASM_WORKERS)
 if (ENVIRONMENT_IS_NODE) {
-  var worker_threads = require('worker_threads');
+  var worker_threads = require('node:worker_threads');
   global.Worker = worker_threads.Worker;
 }
 #endif
 
-#if WASM_WORKERS
-var ENVIRONMENT_IS_WASM_WORKER = globalThis.name == 'em-ww';
+#if AUDIO_WORKLET
+var ENVIRONMENT_IS_AUDIO_WORKLET = !!globalThis.AudioWorkletGlobalScope;
+#endif
 
-#if ENVIRONMENT_MAY_BE_NODE
+#if AUDIO_WORKLET && WASM_WORKERS
+var ENVIRONMENT_IS_WASM_WORKER = {{{ wasmWorkerDetection() }}} || ENVIRONMENT_IS_AUDIO_WORKLET;
+#elif WASM_WORKERS
+var ENVIRONMENT_IS_WASM_WORKER = {{{ wasmWorkerDetection() }}};
+#endif
+
+#if WASM_WORKERS && ENVIRONMENT_MAY_BE_NODE
 if (ENVIRONMENT_IS_NODE) {
   // The way we signal to a worker that it is hosting a pthread is to construct
   // it with a specific name.
-  ENVIRONMENT_IS_WASM_WORKER = worker_threads['workerData'] == 'em-ww'
+  ENVIRONMENT_IS_WASM_WORKER = worker_threads.workerData == 'em-ww'
 }
 #endif
-#endif
-
-#if AUDIO_WORKLET
-var ENVIRONMENT_IS_AUDIO_WORKLET = typeof AudioWorkletGlobalScope !== 'undefined';
-if (ENVIRONMENT_IS_AUDIO_WORKLET) ENVIRONMENT_IS_WASM_WORKER = true;
-#endif
 
 #if ASSERTIONS && ENVIRONMENT_MAY_BE_NODE && ENVIRONMENT_MAY_BE_SHELL
 if (ENVIRONMENT_IS_NODE && ENVIRONMENT_IS_SHELL) {
-  throw 'unclear environment';
+  throw new Error('unclear environment');
 }
 #endif
 
@@ -101,7 +99,7 @@ if (ENVIRONMENT_IS_NODE && ENVIRONMENT_IS_SHELL) {
 var defaultPrint = console.log.bind(console);
 var defaultPrintErr = console.error.bind(console);
 if (ENVIRONMENT_IS_NODE) {
-  var fs = require('fs');
+  var fs = require('node:fs');
   defaultPrint = (...args) => fs.writeSync(1, args.join(' ') + '\n');
   defaultPrintErr = (...args) => fs.writeSync(2, args.join(' ') + '\n');
 }
@@ -116,7 +114,7 @@ var err = (...args) => console.error(...args);
 // compilation is ready. In that callback, call the function run() to start
 // the program.
 function ready() {
-#if MODULARIZE && USE_READY_PROMISE
+#if MODULARIZE
   readyPromiseResolve?.(Module);
 #endif // MODULARIZE
 #if INVOKE_RUN && HAS_MAIN
@@ -132,16 +130,25 @@ function ready() {
 #endif
 }
 
+#if ENVIRONMENT_MAY_BE_NODE
+var isFileURI = (filename) => filename.startsWith('file://');
+var readAsync, readBinary;
+#include "node_shell_read.js"
+#endif
+
+#if ENVIRONMENT_MAY_BE_WORKER || PTHREADS
+var ENVIRONMENT_IS_WORKER = !!globalThis.WorkerGlobalScope;
+#endif
+
 #if PTHREADS
 // MINIMAL_RUNTIME does not support --proxy-to-worker option, so Worker and Pthread environments
 // coincide.
-var ENVIRONMENT_IS_WORKER = typeof WorkerGlobalScope != 'undefined';
-var ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER && self.name?.startsWith('em-pthread');
+var ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER && {{{ pthreadDetection() }}};
 
 #if !MODULARIZE
 // In MODULARIZE mode _scriptName needs to be captured already at the very top of the page immediately when the page is parsed, so it is generated there
 // before the page load. In non-MODULARIZE modes generate it here.
-var _scriptName = typeof document != 'undefined' ? document.currentScript?.src : undefined;
+var _scriptName = globalThis.document?.currentScript?.src;
 #endif
 
 #if ENVIRONMENT_MAY_BE_NODE
@@ -149,7 +156,7 @@ if (ENVIRONMENT_IS_NODE) {
   ENVIRONMENT_IS_WORKER = !worker_threads.isMainThread;
   // Under node we set `workerData` to `em-pthread` to signal that the worker
   // is hosting a pthread.
-  ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER && worker_threads['workerData'] == 'em-pthread'
+  ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER && worker_threads.workerData == 'em-pthread'
 #if !EXPORT_ES6
   _scriptName = __filename;
 #endif
@@ -174,9 +181,9 @@ if (!ENVIRONMENT_IS_PTHREAD) {
 // Wasm or Wasm2JS loading:
 
 if (ENVIRONMENT_IS_NODE) {
-  var fs = require('fs');
+  var fs = require('node:fs');
 #if WASM == 2
-  if (typeof WebAssembly != 'undefined') Module['wasm'] = fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm');
+  if (globalThis.WebAssembly) Module['wasm'] = fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm');
   else eval(fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm.js')+'');
 #else
 #if !WASM2JS
@@ -189,7 +196,7 @@ if (ENVIRONMENT_IS_NODE) {
 #if ENVIRONMENT_MAY_BE_SHELL && ((WASM == 1 && !WASM2JS) || WASM == 2)
 if (ENVIRONMENT_IS_SHELL) {
 #if WASM == 2
-  if (typeof WebAssembly != 'undefined') Module['wasm'] = read('{{{ TARGET_BASENAME }}}.wasm', 'binary');
+  if (globalThis.WebAssembly) Module['wasm'] = read('{{{ TARGET_BASENAME }}}.wasm', 'binary');
   else eval(read('{{{ TARGET_BASENAME }}}.wasm.js')+'');
 #else
 #if !WASM2JS
